'use strict';

var babelHelpers = require('./util/babelHelpers.js');

var React = require('react'),
    cx = require('classnames'),
    dates = require('./util/dates'),
    localizers = require('./util/configuration').locale,
    directions = require('./util/constants').directions,
    Btn = require('./WidgetButton'),
    _ = require('./util/_'),
    compat = require('./util/compat'),
    CustomPropTypes = require('./util/propTypes');

var format = function format(props) {
  return props.monthFormat || localizers.date.formats.month;
};

module.exports = React.createClass({

  displayName: 'YearView',

  mixins: [require('./mixins/WidgetMixin'), require('./mixins/RtlChildContextMixin')],

  propTypes: {
    culture: React.PropTypes.string,
    value: React.PropTypes.instanceOf(Date),
    focused: React.PropTypes.instanceOf(Date),
    min: React.PropTypes.instanceOf(Date),
    max: React.PropTypes.instanceOf(Date),
    onChange: React.PropTypes.func.isRequired,

    monthFormat: CustomPropTypes.dateFormat
  },

  render: function render() {
    var props = _.omit(this.props, ['max', 'min', 'value', 'onChange']),
        months = dates.monthsInYear(dates.year(this.props.focused)),
        rows = _.chunk(months, 4);

    return React.createElement(
      'table',
      babelHelpers._extends({}, props, {
        role: 'grid',
        className: 'rw-calendar-grid rw-nav-view',
        'aria-activedescendant': this._id('_selected_item') }),
      React.createElement(
        'tbody',
        null,
        rows.map(this._row)
      )
    );
  },

  _row: function _row(row, i) {
    var _this = this;

    var id = this._id('_selected_item');

    return React.createElement(
      'tr',
      { key: i, role: 'row' },
      row.map(function (date, i) {
        var focused = dates.eq(date, _this.props.focused, 'month'),
            selected = dates.eq(date, _this.props.value, 'month'),
            currentMonth = dates.eq(date, _this.props.today, 'month');

        return dates.inRange(date, _this.props.min, _this.props.max, 'month') ? React.createElement(
          'td',
          { key: i, role: 'gridcell' },
          React.createElement(
            Btn,
            { onClick: _this.props.onChange.bind(null, date), tabIndex: '-1',
              id: focused ? id : undefined,
              'aria-pressed': selected,
              'aria-disabled': _this.props.disabled || undefined,
              disabled: _this.props.disabled,
              className: cx({
                'rw-state-focus': focused,
                'rw-state-selected': selected,
                'rw-now': currentMonth
              }) },
            localizers.date.format(date, format(_this.props), _this.props.culture)
          )
        ) : React.createElement(
          'td',
          { key: i, className: 'rw-empty-cell', role: 'gridcell' },
          ' '
        );
      })
    );
  }

});